@@ -11,7 +11,7 @@ import logging
 from numpy.linalg import norm
 from scipy.sparse import csc_matrix
 from .trust_region import trust_region, Step
-from .hessian_approximation import HessianApproximation
+from .hessian_approximation import HessianApproximation, HybridUpdate
 from .constants import Options, ExitFlag, DEFAULT_OPTIONS
 from .logging import create_logger
 
@@ -165,8 +165,11 @@ class Optimizer:
                              'update strategy is used), but returned '
                              f'{funout}')
 
-        if self.hessian_update is None:
+        if self.hessian_update is None or isinstance(self.hessian_update,
+                                                     HybridUpdate):
             self.fval, self.grad, self.hess = funout
+            if isinstance(self.hessian_update, HybridUpdate):
+                self.hessian_update.init_mat(len(self.x))
         else:
             if len(funout) == 3:
                 raise ValueError('Cannot use Hessian update with a '
@@ -232,7 +235,7 @@ class Optimizer:
             step = \
                 trust_region(
                     self.x, self.grad, self.hess, scaling,
-                    self.delta, dv, theta, self.lb, self.ub,
+                    self.delta_iter, dv, theta, self.lb, self.ub,
                     subspace_dim=self.get_option(Options.SUBSPACE_DIM),
                     stepback_strategy=self.get_option(Options.STEPBACK_STRAT),
                     refine_stepback=self.get_option(Options.REFINE_STEPBACK),
@@ -243,7 +246,8 @@ class Optimizer:
 
             funout = self.fun(x_new, **self.funargs)
 
-            if self.hessian_update is None:
+            if self.hessian_update is None or isinstance(self.hessian_update,
+                                                         HybridUpdate):
                 fval_new, grad_new, hess_new = funout
             else:
                 fval_new, grad_new = funout
@@ -257,7 +261,7 @@ class Optimizer:
             if self.iteration % 10 == 0:
                 self.log_header()
             self.log_step(accepted, step, fval_new)
-            self.check_convergence(fval_new, x_new, grad_new)
+            self.check_convergence(step, fval_new, grad_new)
 
             # track minimum independently of whether we accept the step or not
             self.track_minimum(x_new, fval_new, grad_new)
@@ -308,9 +312,13 @@ class Optimizer:
         if self.hessian_update is not None:
             self.hessian_update.update(step.s + step.s0,
                                        grad_new - self.grad)
-            self.hess = self.hessian_update.get_mat()
-        else:
+
+        if self.hessian_update is None or \
+                (isinstance(self.hessian_update, HybridUpdate) and
+                 self.iteration < self.hessian_update.switch_iteration):
             self.hess = hess_new
+        else:
+            self.hess = self.hessian_update.get_mat()
         self.check_in_bounds(x_new)
         self.fval = fval_new
         self.x = x_new
@@ -351,7 +359,7 @@ class Optimizer:
             qpval = 0.5 * stepsx.dot(dv * np.abs(grad) * stepsx)
             self.tr_ratio = (fval + qpval - self.fval) / step.qpval
 
-            interior_solution = nsx < self.delta * 0.9
+            interior_solution = nsx < self.delta_iter * 0.9
 
             # values as proposed in algorithm 4.1 in Nocedal & Wright
             if self.tr_ratio >= self.get_option(Options.ETA) \
@@ -366,14 +374,17 @@ class Optimizer:
                 ])
             return self.tr_ratio > 0.0
 
-    def check_convergence(self, fval, x, grad) -> None:
+    def check_convergence(self, step: Step, fval: float,
+                          grad: np.ndarray) -> None:
         """
         Check whether optimization has converged.
 
+        :param step:
+            update to optimization variables
+
         :param fval:
             updated objective function value
-        :param x:
-            updated optimization variables
+
         :param grad:
             updated objective function gradient
         """
@@ -381,13 +392,15 @@ class Optimizer:
 
         fatol = self.get_option(Options.FATOL)
         frtol = self.get_option(Options.FRTOL)
-        xatol = self.get_option(Options.XATOL)
-        xrtol = self.get_option(Options.XRTOL)
+        xtol = self.get_option(Options.XTOL)
         gatol = self.get_option(Options.GATOL)
         grtol = self.get_option(Options.GRTOL)
         gnorm = norm(grad)
+        stepsx = step.ss + step.ss0
+        nsx = norm(stepsx)
 
-        if np.isclose(fval, self.fval, atol=fatol, rtol=frtol):
+        if self.delta <= self.delta_iter and \
+                np.abs(fval - self.fval) < fatol + frtol*np.abs(self.fval):
             self.exitflag = ExitFlag.FTOL
             self.logger.warning(
                 'Stopping as function difference '
@@ -396,11 +409,12 @@ class Optimizer:
             )
             converged = True
 
-        elif np.isclose(x, self.x, atol=xatol, rtol=xrtol).all():
+        elif self.iteration > 1 and nsx < xtol:
             self.exitflag = ExitFlag.XTOL
             self.logger.warning(
-                'Stopping as step was smaller than specified tolerances ('
-                f'atol={xatol:.2E}, rtol={xrtol:.2E})'
+                'Stopping as norm of step '
+                f'{nsx} was smaller than specified '
+                f'tolerance (tol={xtol:.2E})'
             )
             converged = True
 
@@ -455,6 +469,15 @@ class Optimizer:
             )
             return False
 
+        if self.delta < np.spacing(1):
+            self.exitflag = ExitFlag.DELTA_TOO_SMALL
+            self.logger.warning(
+                f'Stopping as trust region radius {self.delta:.2E} is '
+                f'smaller '
+                'than machine precision.'
+            )
+            return False
+
         return True
 
     def make_non_degenerate(self, eps=1e2 * np.spacing(1)) -> None:
